#----------------------------------------------------------------------------
# Name :          randomPushPull
# Description :   PushPull selected faces by random amount within a range
# Author :        TIG (c)
# Usage :         Select Plugins >> Random PushPull
#				  Enter Max/Min range in current units, defaults=100 and 0
# Version:        1.0 first issue. looong_ago...
#				  2.0 reissue after Jim's tweaks & other adjustments. 20110707
#				  2.1 range error corrected. 20110707
#			      2.2 negative pp values allowed. 20110707
#				  2.3 negative values booboo corrected. 20110707
#				  2.4 selection passes to pushpulled faces. 20110707
#----------------------------------------------------------------------------

require 'sketchup.rb'

module TIG
  @ppmax=nil if not @ppmax
  @ppmin=nil if not @ppmin
  def self.randomPushPull()
    model=Sketchup.active_model
    if Sketchup.version.split(".")[0].to_i >= 7
	  model.start_operation("randomPushPull",true)
    else
	  model.start_operation("randomPushPull")
    end
	ents=model.active_entities
    ss=model.selection
    selection=ss.to_a.select{|e|e.class==Sketchup::Face}
    if ss.empty? 
	  UI.messagebox("No Suitable Selection !")
	  return nil
    end
	ss.clear
    if not @ppmax or not @ppmin
	  @ppmax="100".to_l
	  @ppmin="0".to_l
    end
    prompts = ["Max: ","Min: "]
    values =  [@ppmax,@ppmin]
    results = UI.inputbox(prompts,values,"Random PushPull Range")
    return nil if not results
    @ppmax=results[0].to_l
    @ppmin=results[1].to_l
	dif=@ppmax-@ppmin
	data=[]
    selection.each{|face|
	  rn=rand(1000001)
	  if rn!=0
	    pp=dif*(rn/1000000.0)
		if pp.abs > dif.abs
	      pp=dif
	    end
	  else
	    pp=0
	  end
	  di=pp+@ppmin
	  data << [face.area, [face.vertices[0].position.offset(face.normal, di), face.normal]]
	  face.pushpull(di) if di!=0
	}
	faces=[]; ents.each{|e|faces << e if e.class==Sketchup::Face}
	faces.each{|face|
	  data.each{|e|
	    ss.add(face) if face.area-e[0].abs < 0.000001 and face.vertices[0].position==e[1][0] and (face.normal==e[1][1] or face.normal.reverse==e[1][1])
	  }
	}
    model.commit_operation
  end
end

#----------------------------------------------------------------------------
# add menu items
if not file_loaded?(File.basename(__FILE__))
    UI.menu("Plugins").add_item("Random PushPull"){TIG.randomPushPull()}
end
file_loaded(File.basename(__FILE__))
#----------------------------------------------------------------------------

